@@ -1,7 +1,5 @@
 """ ImageContainer tests """
-# pylint: disable=redefined-outer-name
-# pylint: disable=protected-access
-
+# pylint: disable=redefined-outer-name,protected-access,duplicate-code
 import os
 from typing import List
 
@@ -91,8 +89,8 @@ NIFTI_AFFINE = np.array(
 
 @pytest.fixture
 def nifti_image_containers_a() -> List[NiftiImageContainer]:
-    """ Returns a list of NiftiImageContainers. The first items uses
-    a Nifti1Image. The second item uses a Nifti2Image """
+    """Returns a list of NiftiImageContainers. The first items uses
+    a Nifti1Image. The second item uses a Nifti2Image"""
 
     containers = []
     for image_type in [nib.Nifti1Image, nib.Nifti2Image]:
@@ -118,7 +116,7 @@ def nifti_image_containers_a() -> List[NiftiImageContainer]:
 
 
 def test_nifti_image_container_image(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the image is returned from a NiftiImageContainer """
 
@@ -128,7 +126,7 @@ def test_nifti_image_container_image(
 
 
 def test_nifti_image_container_shape(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the shape is correctly returned from a NiftiImageContainer """
 
@@ -137,7 +135,7 @@ def test_nifti_image_container_shape(
 
 
 def test_nifti_image_container_affine(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the image is returned from a NiftiImageContainer """
 
@@ -156,7 +154,7 @@ def test_nifti_image_container_affine(
 
 
 def test_nifti_image_container_time_units(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the correct time units are returned from a NiftiImageContainer """
 
@@ -165,7 +163,7 @@ def test_nifti_image_container_time_units(
 
 
 def test_nifti_image_container_space_units(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the correct space_units are returned from a NiftiImageContainer """
 
@@ -174,7 +172,7 @@ def test_nifti_image_container_space_units(
 
 
 def test_nifti_image_xyzt_units_reading(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test the xyzt_units are interpreted properly """
     for image_container in nifti_image_containers_a:
@@ -193,7 +191,7 @@ def test_nifti_image_xyzt_units_reading(
 
 
 def test_nifti_image_container_voxel_size_mm(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the correct voxel size is returned from a NiftiImageContainer """
 
@@ -212,8 +210,23 @@ def test_nifti_image_container_voxel_size_mm(
         )
 
 
+def test_nifti_image_container_voxel_size_mm_setter(
+    nifti_image_containers_a: NiftiImageContainer,
+):
+    """ Test that the correct voxel size is set on a NiftiImageContainer """
+    for image_container in nifti_image_containers_a:
+        # voxel size is 2x2x2mm
+        image_container.voxel_size_mm = [3.0, 2.0, 1.0]
+        image_container.space_units = UNITS_METERS
+
+        # Just test that we have 3x2x1m reported in mm
+        np.testing.assert_array_almost_equal(
+            image_container.voxel_size_mm, np.array([3000.0, 2000.0, 1000.0])
+        )
+
+
 def test_nifti_image_container_time_step_seconds(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the correct time step is returned from a NiftiImageContainer """
 
@@ -226,8 +239,20 @@ def test_nifti_image_container_time_step_seconds(
         assert image_container.time_step_seconds == 2e9
 
 
+def test_nifti_image_container_time_step_seconds_setter(
+    nifti_image_containers_a: List[NiftiImageContainer],
+):
+    """ Test that the correct time step is get on a NiftiImageContainer """
+    for image_container in nifti_image_containers_a:
+
+        image_container.time_step_seconds = 10.0
+        image_container.time_units = UNITS_MILLISECONDS
+
+        np.testing.assert_almost_equal(image_container.time_step_seconds, 10000.0)
+
+
 def test_nifti_image_container_has_header(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test that the nifti header is returned from a NiftiImageContainer"""
 
@@ -293,8 +318,8 @@ def test_numpy_image_container_space_units(numpy_image_container):
     assert numpy_image_container.space_units == UNITS_MILLIMETERS
 
 
-def test_numpy_image_container_voxel_size_mm(
-    numpy_image_container: NumpyImageContainer
+def test_numpy_image_container_voxel_size_mm_getter(
+    numpy_image_container: NumpyImageContainer,
 ):
     """ Test that the correct voxel size is returned from a NumpyImageContainer """
 
@@ -312,8 +337,22 @@ def test_numpy_image_container_voxel_size_mm(
     )
 
 
-def test_numpy_image_container_time_step_seconds(
-    numpy_image_container: NumpyImageContainer
+def test_numpy_image_container_voxel_size_mm_setter(
+    numpy_image_container: NumpyImageContainer,
+):
+    """ Test that the correct voxel size is set on a NumpyImageContainer """
+    # voxel size is 2x2x2mm
+    numpy_image_container.voxel_size_mm = [3.0, 2.0, 1.0]
+    numpy_image_container.space_units = UNITS_METERS
+
+    # Just test that we have 3x2x1m reported in mm
+    np.testing.assert_array_almost_equal(
+        numpy_image_container.voxel_size_mm, np.array([3000.0, 2000.0, 1000.0])
+    )
+
+
+def test_numpy_image_container_time_step_seconds_getter(
+    numpy_image_container: NumpyImageContainer,
 ):
     """ Test that the correct time step is returned from a NumpyImageContainer """
 
@@ -325,6 +364,17 @@ def test_numpy_image_container_time_step_seconds(
     assert numpy_image_container.time_step_seconds == 2e9
 
 
+def test_numpy_image_container_time_step_seconds_setter(
+    numpy_image_container: NumpyImageContainer,
+):
+    """ Test that the correct time step is get on a NumpyImageContainer """
+
+    numpy_image_container.time_step_seconds = 10.0
+    numpy_image_container.time_units = UNITS_MILLISECONDS
+
+    np.testing.assert_almost_equal(numpy_image_container.time_step_seconds, 10000.0)
+
+
 def test_numpy_image_container_has_header(numpy_image_container: NumpyImageContainer):
     """ Test that a header is not returned from a NumpyImageContainer """
 
@@ -349,7 +399,7 @@ def test_image_container_unexpected_arguments():
 
 # SPATIAL_DOMAIN tests
 def test_image_container_spatial_domain_initialisation():
-    """ Check that passing a string not in SPATIAL_DOMAIN or INVERSE_DOMAIN to
+    """Check that passing a string not in SPATIAL_DOMAIN or INVERSE_DOMAIN to
     data_domain raises an exception ( and vice versa )"""
     with pytest.raises(ValueError):
         NumpyImageContainer(image=np.zeros((3, 3, 3)), data_domain="foobar")
@@ -373,8 +423,8 @@ def test_image_container_image_type_bad_initialisation():
 
 
 def test_image_container_image_type_good_bad_initialisation():
-    """ Check initialising image containers with image types creates the correct defaults
-    and if an image_type is supplied, it is correctly validated """
+    """Check initialising image containers with image types creates the correct defaults
+    and if an image_type is supplied, it is correctly validated"""
     for dtype in [
         np.int8,
         np.int16,
@@ -470,6 +520,7 @@ def general_image_container_clone_tests(
     )
 
     assert image_container.time_step_seconds == cloned_image_container.time_step_seconds
+    assert image_container.metadata == cloned_image_container.metadata
 
 
 def test_numpy_image_container_clone():
@@ -490,7 +541,7 @@ def test_numpy_image_container_clone():
 
 
 def test_nifti_image_container_clone(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Check that the NIFTI image container is cloned correctly """
 
@@ -509,7 +560,7 @@ def test_nifti_image_container_clone(
 
 
 def test_numpy_image_container_image_properties(
-    numpy_image_container: NumpyImageContainer
+    numpy_image_container: NumpyImageContainer,
 ):
     """ Test the numpy image container image setter/getter """
     new_image = np.ones(shape=(4, 4, 4)) * 10
@@ -520,7 +571,7 @@ def test_numpy_image_container_image_properties(
 
 
 def test_nifti_image_container_image_properties(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
     """ Test the nifti image containers image setter/getter """
     for image_container in nifti_image_containers_a:
@@ -536,10 +587,10 @@ def test_nifti_image_container_image_properties(
 
 
 def test_nifti_image_container_image_set_different_dtype(
-    nifti_image_containers_a: List[NiftiImageContainer]
+    nifti_image_containers_a: List[NiftiImageContainer],
 ):
-    """ Check that setting a nifti image container's image data
-    using a different dtype to the original updates the nifti header correctly """
+    """Check that setting a nifti image container's image data
+    using a different dtype to the original updates the nifti header correctly"""
 
     for image_container in nifti_image_containers_a:
         new_image = np.ones(shape=(4, 4, 4), dtype=np.int8)
@@ -577,3 +628,99 @@ def test_nifti_image_container_image_set_different_dtype(
         new_image = np.ones(shape=(4, 4, 4), dtype=np.float64)
         image_container.image = new_image
         assert image_container.header["datatype"] == 64
+
+
+def test_image_container_metadata_init():
+    """ Test the metadata initialisation on the Image Container classes """
+    numpy = NumpyImageContainer(image=np.ones((1, 1, 1)), affine=np.eye(4))
+    assert numpy.metadata == {}
+    nifti = NiftiImageContainer(
+        nifti_img=nib.Nifti1Image(np.ones((1, 1, 1)), affine=np.eye(4))
+    )
+    assert nifti.metadata == {}
+
+    numpy = NumpyImageContainer(
+        image=np.ones((1, 1, 1)), affine=np.eye(4), metadata={"foo": "bar"}
+    )
+    assert numpy.metadata == {"foo": "bar"}
+    nifti = NiftiImageContainer(
+        nifti_img=nib.Nifti1Image(np.ones((1, 1, 1)), affine=np.eye(4)),
+        metadata={"bar": "foo"},
+    )
+    assert nifti.metadata == {"bar": "foo"}
+
+    with pytest.raises(TypeError):
+        NumpyImageContainer(
+            image=np.ones((1, 1, 1)), affine=np.eye(4), metadata=1
+        )  # non-dict
+    with pytest.raises(TypeError):
+        NiftiImageContainer(
+            nifti_img=nib.Nifti1Image(np.ones((1, 1, 1)), affine=np.eye(4)),
+            metadata="foobar",  # non-dict
+        )
+
+
+def test_image_container_metadata_get_set():
+    """ Test the Image Container metadata getting and setting """
+    numpy = NumpyImageContainer(image=np.ones((1, 1, 1)), affine=np.eye(4))
+    assert numpy.metadata == {}
+    numpy.metadata = {"one": 1, "two": 2}
+    assert numpy.metadata == {"one": 1, "two": 2}
+    numpy.metadata["three"] = 3
+    assert numpy.metadata == {"one": 1, "two": 2, "three": 3}
+    with pytest.raises(TypeError):
+        numpy.metadata = "not a dict"
+
+
+def test_nifti_to_numpy(nifti_image_containers_a: List[NiftiImageContainer]):
+    """ Check the as_numpy()/as_nifti() functionality works correctly on a nifti container """
+    for image_container in nifti_image_containers_a:
+        new_image_container = image_container.as_numpy()
+        for new_image_container in [
+            image_container.as_numpy(),
+            image_container.as_nifti(),
+        ]:
+            np.testing.assert_array_equal(
+                new_image_container.image, image_container.image
+            )
+            np.testing.assert_array_equal(
+                new_image_container.affine, image_container.affine
+            )
+            assert new_image_container.data_domain == image_container.data_domain
+            assert new_image_container.image_type == image_container.image_type
+            assert new_image_container.metadata == image_container.metadata
+            assert new_image_container.space_units == image_container.space_units
+            assert new_image_container.time_units == image_container.time_units
+            np.testing.assert_array_equal(
+                new_image_container.voxel_size_mm, image_container.voxel_size_mm
+            )
+            assert (
+                new_image_container.time_step_seconds
+                == image_container.time_step_seconds
+            )
+
+
+def test_numpy_to_nifti(numpy_image_container: NumpyImageContainer):
+    """ Check the as_nifti functionality works correctly on a nifti container """
+    for new_image_container in [
+        numpy_image_container.as_numpy(),
+        numpy_image_container.as_nifti(),
+    ]:
+        np.testing.assert_array_equal(
+            new_image_container.image, numpy_image_container.image
+        )
+        np.testing.assert_array_equal(
+            new_image_container.affine, numpy_image_container.affine
+        )
+        assert new_image_container.data_domain == numpy_image_container.data_domain
+        assert new_image_container.image_type == numpy_image_container.image_type
+        assert new_image_container.metadata == numpy_image_container.metadata
+        assert new_image_container.space_units == numpy_image_container.space_units
+        assert new_image_container.time_units == numpy_image_container.time_units
+        np.testing.assert_array_equal(
+            new_image_container.voxel_size_mm, numpy_image_container.voxel_size_mm
+        )
+        assert (
+            new_image_container.time_step_seconds
+            == numpy_image_container.time_step_seconds
+        )
